/*   COMICS ORGANIZER      */
/*   Editing routines      */

#include <exec/types.h>
#include <exec/memory.h>
#include <libraries/dos.h>
#include <intuition/intuition.h>
#include <graphics/gfxbase.h>
#include <proto/exec.h>
#include <proto/dos.h>

#include <stdio.h>

#include <libraries/reqtools.h>
#include <proto/reqtools.h>

#include "comics.h"

/* Variables & structure definition ------------------------ */

ComicNodePtr pushstack[] =
{	NULL,NULL,NULL,NULL,NULL,
	NULL,NULL,NULL,NULL,NULL   };

int	stackpos = 0;	/*  Current position in stack */

struct	ComicNode	ClipNode;     /*  "Clipboard"  */
BOOL	CLIPEXISTS = FALSE;

extern struct Window	*MainWindow;
extern struct Window	*IndexWindow;
extern struct Screen	*ComicScreen;
extern struct ComicFile	Comics;
extern ComicNodePtr	head,current,previous;
extern int		CurIssue;
extern struct ComicNode	WorkNode;
extern BOOL		IssueInd,TitleInd,ReallyChanged;
extern struct Gadget	IndexUp;


/*  Routine prototypes ------------------------------------ */

void PushComic(),PopComic();
void RemoveComic(),CutComic(),DeleteComic(),PasteComic();
void DeleteRange();

void UpdateRange(),UpdateComic(),UpdateDate(ComicNodePtr);


/*  External routines ------------------------------------- */

extern void SaveComic(),NewComic();    /* Import from list.c */
extern void DisplayComic();
extern void CopyNode();
extern void ConvertStr();
extern ComicNodePtr NEW();

/*  STACK ROUTINES ----------------------------------------- */

void PushComic()
{
	if (Comics.Changed)	SaveComic();

	if (current == NULL)	return;
	
	if (pushstack[stackpos] != NULL)
	{	stackpos++;
		if (stackpos > 9)	stackpos = 0;    }

	pushstack[stackpos] = current;
	SetWindowTitles(MainWindow,-1,"PUSHED");
}

void PopComic()
{
	register ComicNodePtr	cur;
	
	if (Comics.Changed)	SaveComic();
	
	if (pushstack[stackpos])
	{
		current = pushstack[stackpos--];
		if (stackpos < 0)	stackpos = 0;
		/*  Find what issue this is  */
		CurIssue = 0;
		cur = head;
		while ( (cur != NULL) && (cur != current) )
		{
			cur = cur->next;
			CurIssue++;
		}
		pushstack[stackpos] = NULL;
	}
	DisplayComic();
}


/*  CUTTING & PASTING ------------------------------------------- */

void RemoveComic()
/*  Actual routine to remove current comic from the list  */
{
    ComicNodePtr	prevpointer;
    register int i;
	
    if (current != NULL)	/*  Make sure it exists!  */
    {
			
	/*  Make sure it hasn't been marked  */
	for (i=0;i <= 9;i++)
	{
		if (pushstack[stackpos] == current)
			pushstack[stackpos] = NULL;
	}
	
	if (previous == current)	previous = NULL;
	
	if (current->prev == NULL)   /* Erasing first node in list */
	{
		head = current->next;
		if (head != NULL)		head->prev = NULL;
		FreeMem(current,sizeof(struct ComicNode));
		current = head;
		if (current)	CurIssue = 1;
	}
	else
	{
		prevpointer = current->prev;
		prevpointer->next = current->next;
		CurIssue--;
		if (current->next != NULL)
		{
			current->next->prev = prevpointer;
			prevpointer = current->next;
			CurIssue++;
		}
		FreeMem(current,sizeof(struct ComicNode));
		current = prevpointer;
	}
	Comics.TotalIssues--;
    }
}

void DeleteComic()
{
	RemoveComic();
	
	Comics.Changed = FALSE;
	Comics.WorkSaved = FALSE;
	DisplayComic();
}

void CutComic()
{
	if (current != NULL)
	{
		CopyNode (&ClipNode,current);
		ClipNode.next = NULL;
		ClipNode.prev = NULL;
		CLIPEXISTS = TRUE;
		
		DeleteComic();
	}
}

void PasteComic()
{
	if (Comics.Changed)	SaveComic();
	
	if (CLIPEXISTS)
	{	NewComic();	
		CopyNode(&WorkNode,&ClipNode);
		DisplayComic();
		Comics.Changed = TRUE;
		ReallyChanged = TRUE;  /* Might have changed title/index */
		Comics.WorkSaved = FALSE;	}
}


void DeleteRange()
{
	long numberofissues = 1L;
	register int counter = 0;
	ULONG response;
	ComicNodePtr cur,oldprev,oldnext;
	
	if (current == NULL)           /* Don't need to remove from list */
	{	ClearNode(&WorkNode);
		current = previous;
		DisplayComic();
		return;			}
	
	if (!(rtGetLong(&numberofissues,"Delete how many issues?",NULL,NULL) ))
		return;
	
	if (numberofissues > (long) Comics.TotalIssues)
		numberofissues = (long) Comics.TotalIssues;
	
	response = rtEZRequest ("Delete %ld issues?","DELETE|Cancel",NULL,NULL,numberofissues);
	if (response == FALSE)	return;
	
	counter = (int) numberofissues;
	oldprev = current->prev;
	cur = current;
	while ( (cur != NULL) && (counter > 0) )
	{
		Comics.TotalIssues--;
		counter--;
		oldnext = cur->next;
		FreeMem(cur,sizeof(struct ComicNode));
		cur = oldnext;
	}
	/*  Re-link pointers  */
	
	if (oldprev == NULL)   /* Started with first node in list */
	{
		head = oldnext;
		oldnext->prev = NULL;
		current = head;
		CurIssue = 1;
		if (head == NULL)	CurIssue = 0;
	}
	else
	{
		oldprev->next = oldnext;
		oldnext->prev = oldprev;
		if (oldnext != NULL) 	current = oldnext;
		else
		{			current = oldprev;
					CurIssue--;	}
	}
	Comics.WorkSaved = FALSE;
	Comics.Changed	 = TRUE;
	
	previous = NULL;
	DisplayComic();
}

/* UPDATING ---------------------------------------------- */

void UpdateRange()
{
	long startissue,endissue;
	int oldCurIssue;
	ComicNodePtr oldcurrent,oldnext;
	
	if (Comics.Changed)	SaveComic();
	oldcurrent = current;
	oldnext = current->next;
	oldCurIssue = CurIssue;

	startissue = (long) current->Issue;
	endissue = startissue;
	
	if (!(rtGetLong(&endissue,"Ending issue #?",NULL,NULL) ))
		return;
	
	while ( (startissue < endissue) && (current != NULL) )
	{
		previous = current;
		current = NEW();
		if (current)
		{
			current->next = NULL;
			current->prev = previous;
			previous->next = current;
			UpdateComic();
			Comics.TotalIssues++;
			startissue++;
		}
	}
	
	current->next = oldnext;
	oldnext->prev = current;
	current = oldcurrent;
	CurIssue = oldCurIssue;
	Comics.Changed = FALSE;
	Comics.WorkSaved = FALSE;
	ReallyChanged = FALSE;
	DisplayComic();
}
	
void UpdateComic()
{
	ComicNodePtr	ptr;
	
	if (previous == NULL)
	{
		rtEZRequest("No previous issue to update!","Cancel",NULL,NULL);
	}
	else
	{	
		if (current)	ptr = current;
		else		ptr = &WorkNode;
		
		strcpy(ptr->Title,previous->Title);
		ptr->Issue = previous->Issue + 1;
		strcpy(ptr->Artist,previous->Artist);
		strcpy(ptr->Condition,previous->Condition);
		strcpy(ptr->Publisher,previous->Publisher);
		strcpy(ptr->Format,previous->Format);
		ptr->Copies = previous->Copies;
	
		if (Comics.Update & UPDATEDATE)
			UpdateDate(ptr);
		if (Comics.Update & UPDATEPRICES)
		{
			ptr->Cover = previous->Cover;
			ptr->Purchase = previous->Purchase;
			ptr->Current = previous->Current;
			ptr->Profit = previous->Profit;
		}
		if (Comics.Update & UPDATENOTES)
			strcpy (ptr->Notes,previous->Notes);
	
		Comics.Changed = TRUE;
		Comics.WorkSaved = FALSE;
		DisplayComic();
	}
}


void UpdateDate(ptr)
ComicNodePtr ptr;
{
	char months[37];
	char themonth[9],yearstr[9],finalstr[9];
	register int  i, theyear;
	BOOL success;
	
	strcpy (months,"JANFEBMARAPRMAYJUNJULAUGSEPOCTNOVDEC");
	
	strncpy (themonth,previous->Date,3);  /* Copy first 3 chars */
	ConvertStr(themonth);
	
	/*  Find the 3 chars in months string */
	i = 0;
	success = FALSE;
	while ( ( i < 36 ) && ( success == FALSE ) )
	{
		if (strncmp(themonth,&months[i],3) == 0)
			success = TRUE;
		i += 3;
	}
	if (success)
	{
		theyear = atoi(previous->Date + 4);
		if (i > 33)  /* month was December, increment year */
		{
			i = 0;
			theyear++;
		}
		
		sprintf(yearstr," %d",theyear);
		yearstr[3] = 0;  /* truncate after 3 chars */
		strncpy(finalstr,&months[i],3);
		finalstr[3] = 0;
		strcat(finalstr,yearstr);
		finalstr[6] = 0;
		strcpy(ptr->Date,finalstr);
	}
}