/*  COMICS ORGANIZER              */
/*  List Management               */

#include <exec/types.h>
#include <exec/memory.h>
#include <libraries/dos.h>
#include <intuition/intuition.h>
#include <proto/exec.h>
#include <proto/dos.h>

#include <stdio.h>

#include <libraries/reqtools.h>
#include <proto/reqtools.h>

#include <libraries/mathffp.h>

#include "comics.h"


/*   GLOBAL VARIABLES  -------------------------------- */

struct ComicNode 	WorkNode;  /*  Holds information of current node */

ComicNodePtr head = NULL;	      /*  First position in list    */
ComicNodePtr current = NULL;          /*  Current Position in List  */
ComicNodePtr previous = NULL;	      /*  Last entry                */
 
extern struct ComicFile Comics;	      /*  Contains all the file vars */
int	CurIssue = 0;

char		windowtitle[100];

BOOL ReallyChanged = FALSE;	/* This comic needs to be repositioned? */
				/* (Have changed Issue # or Title) */
				
/*  Prototypes ----------------------------------------------------- */

void CopyNode(ComicNodePtr,ComicNodePtr);
void ClearNode(ComicNodePtr);
ComicNodePtr NEW();
void AddNode();
void SaveComic();
void DisposeList();
void DisplayComic();
void ReadInfo(USHORT);
void Previous(), Next(), First(), Last();
void PrevTitle(), NextTitle();
void NewComic();
void UpdateProfit();
void strunc(char *,float);
void UpdateMainProp(), TranslateMainProp();
void GetPublisher(), GetFormat(), GetCondition();
void InputRequester(USHORT,USHORT,char *,char *);
ULONG BuildRequester(USHORT,USHORT,char *,BOOL);
void ConvertStr(char *);
void ActivateNum(USHORT);

extern void UpdateComic(),RemoveComic();	/* Import from edit.c */


/* Info for main window and screen ------------------------ */
extern struct Window	*MainWindow;
extern struct Screen	*ComicScreen;

/* External Gadget Info ----------------------------------- */

extern struct StringInfo CopiesGad_info;
extern struct StringInfo NotesGad_info;
extern struct StringInfo ProfitGad_info;
extern struct StringInfo CurrentGad_info;
extern struct StringInfo PurchaseGad_info;
extern struct StringInfo CoverGad_info;
extern struct StringInfo ArtistGad_info;
extern struct StringInfo DateGad_info;
extern struct StringInfo IssueGad_info;
extern struct StringInfo TitleGad_info;
extern struct Gadget	 FirstGad;
extern struct Gadget	 TitleGad,IssueGad,DateGad,ArtistGad,CoverGad,
			 PurchaseGad,CurrentGad,NotesGad,CopiesGad;
extern struct Gadget	 MainProp;
extern struct PropInfo	 MainProp_info;

/*  External data for other gadgets        */

extern struct IntuiText SubItemsText[];

extern UBYTE FormatText[];
extern UBYTE ConditionText[];
extern UBYTE PublishText[];


/*   LIST HANDLING FUNCTIONS -------------------------------------- */

void SaveComic()
/*  If the comic is already in the list, update its info.  Else, add it
    to the list   */
{
	if (current != NULL)
	{
		if (!ReallyChanged)
			CopyNode (current,&WorkNode);
		else		/*  Changed title or issue, reposition */
		{	RemoveComic();
			AddNode();		}
	}
	else
	{
		AddNode();
	}
	
	previous = current;
	if (current != NULL)
	{
		Comics.Changed = FALSE; /* Couldn't create it */
		ReallyChanged = FALSE;
	}
}

void AddNode()
{
	ComicNodePtr	cur,prev;
	char	curtitle[100],testtitle[100];
	
	current = NEW();
	if (!current)
	{
		DisplayBeep(ComicScreen);
	}
	else
	{
		/*  Copy the Work Node's data into this new node  */
		CopyNode(current,&WorkNode);
		
		CurIssue = 1;
		cur = head;
		prev = NULL;
	
		/*  Find correct position for title              */
		strcpy (curtitle,current->Title);
		ConvertStr(curtitle);
		strcpy (testtitle,cur->Title);
		ConvertStr(testtitle);
		
		while ( (cur != NULL) && 
			(strcmp (curtitle,testtitle) > 0) )
		{
			prev = cur;
			cur = cur->next;
			CurIssue++;
			strcpy(testtitle,cur->Title);
			ConvertStr(testtitle);
		}
		
		/*  Find correct position for issue		*/
		while ( (cur !=NULL) &&
			(current->Issue > cur->Issue) &&
			(strcmp (curtitle,testtitle) == 0) )
		{
			prev = cur;
			cur = cur->next;
			CurIssue++;
			strcpy(testtitle,cur->Title);
			ConvertStr(testtitle);
		}
		
		if ( (cur == NULL) && (prev == NULL) )  /* List is empty */
		{
			current->next = NULL;
			current->prev = NULL;
			head = current;
		}
		else if (cur == head)		/* Entered as first */
		{
			current->next = head;
			current->prev = NULL;
			head->prev = current;
			head = current;
		}
		else				/* Entered as normal */
		{
			current->prev = prev;
			current->next = prev->next;
			prev->next = current;
			current->next->prev = current;
		}
		Comics.TotalIssues += 1;
	}
}

void ConvertStr(theinput)
char *theinput;
/*  Changes a string to all caps for more accurate sorting, removes the
    "The" from a title if it exists.  */
{
	char c,thestring[TITLELEN];
	register int counter = 0,counter2 = 0;
	int thelength;
	
	thelength = strlen(theinput);
	strcpy (thestring,theinput);
	c = thestring[0];
	while (c != '\0')
	{
		thestring[counter++] = toupper(c);
		c = thestring[counter];
	}
	if (Comics.ChangeThe)
	{
		/* remove all "the"'s and "a"'s */
		for (counter = 0;counter <= thelength;counter++)
		{
			if (strncmp(&thestring[counter],"THE ",4) == 0)
				counter += 4;
			else if (strncmp(&thestring[counter],"A ",2) == 0)
				counter += 2;
			*(theinput + counter2++) = thestring[counter];
		}
		*(theinput + counter2) = '\0';
	}
}
	

ComicNodePtr NEW()
/*  Allocates memory for a new node, returns a pointer to it  */
{
	ComicNodePtr newpointer;
	
	/*  equivalent to newpointer = malloc(sizeof(struct ComicNode)) */
	newpointer = (ComicNodePtr) AllocMem(sizeof(struct ComicNode),MEMF_CLEAR);
	if (!newpointer)
	{
		rtEZRequest("ERROR: Couldn't create a new node!\nOut of memory?","ABORT",NULL,NULL);
	}
	return (newpointer);
}
			

void DisposeList()
/*  Frees the memory for every node in the list    */
{
	ComicNodePtr cur,nxt;
	
	cur = head;
	while (cur != NULL)
	{
		nxt = cur->next;
		FreeMem(cur,sizeof(struct ComicNode));
		cur = nxt;
	}

	/*  Reset variables in case this is called by NewFile routine */
	Comics.TotalIssues = 0;
	CurIssue = 0;
	head = NULL;
	current = NULL;
	previous = NULL;
	strcpy (Comics.FileName,"Untitled");
	strcpy (Comics.FilePath,"");
	Comics.Changed = FALSE;
	Comics.WorkSaved = TRUE;
}


/* STRING GADGET READING/DISPLAY ---------------------------------- */

void DisplayComic()
/*  Fills all the string gadgets with the contents of the current node   */
{
	if (current)	CopyNode(&WorkNode,current);
	
	strcpy (TitleGad_info.Buffer,WorkNode.Title);
	sprintf (IssueGad_info.Buffer,"%d",WorkNode.Issue);
	strcpy (DateGad_info.Buffer,WorkNode.Date);
	strcpy (ArtistGad_info.Buffer,WorkNode.Artist);
	strunc (CoverGad_info.Buffer,WorkNode.Cover);
	strunc (PurchaseGad_info.Buffer,WorkNode.Purchase);
	strunc (CurrentGad_info.Buffer,WorkNode.Current);
	strunc (ProfitGad_info.Buffer,WorkNode.Profit);
	strcpy (NotesGad_info.Buffer,WorkNode.Notes);
	sprintf (CopiesGad_info.Buffer,"%d",WorkNode.Copies);
	strcpy (ConditionText,WorkNode.Condition);
	strcpy (FormatText,WorkNode.Format);
	strcpy (PublishText,WorkNode.Publisher);
	
	RefreshGadgets(&FirstGad,MainWindow,NULL);
	UpdateMainProp();
		
	if (current == NULL)
		sprintf(&windowtitle[0],"%s  NEW COMIC of %d",Comics.FileName,Comics.TotalIssues);
	else
		sprintf(&windowtitle[0],"%s  %d of %d",Comics.FileName,CurIssue,Comics.TotalIssues);
	
	SetWindowTitles(MainWindow,&windowtitle[0],-1);
}

void strunc (thestring,thenum)
char *thestring;
float thenum;
/*  Puts a floating point number into a string, truncating two digits 
    after the decimal.    */
{
	int position = 0;
	
	sprintf (thestring,"%f",thenum);
	
	while ( *(thestring + position) != '.' &&
		*(thestring + position) != '\0'  )
	{
		position++;
	}
	if ( *(thestring + position    ) != '\0' &&
	     *(thestring + position + 1) != '\0' &&
	     *(thestring + position + 2) != '\0' )
	{
		*(thestring + position + 3) = '\0';
	}
}


void ReadInfo(gadid)
USHORT gadid;
/*  Puts the information in string gadget gadid into the proper position
    in Work Node.
    Automatically advances cursor to the next gadget in the gadget list. */
{
   switch (gadid)
   {
	case 7  : if ( strlen(TitleGad_info.Buffer) )
		  /* if title contains a `, update the comic */
		  {  strcpy(WorkNode.Title,TitleGad_info.Buffer);
		     if ( strchr(WorkNode.Title,'`') )
		  	UpdateComic();
		     else
		     {  ActivateGadget(&IssueGad,MainWindow,NULL);
		        Comics.Changed = TRUE;
		        Comics.WorkSaved = FALSE;
			ReallyChanged = TRUE;  }
		  }
		  break;
	case 8  : if ( IssueGad_info.LongInt )
		  {  WorkNode.Issue = (int) IssueGad_info.LongInt;
		     ActivateGadget(&DateGad,MainWindow,NULL);
		     Comics.Changed = TRUE;
		     Comics.WorkSaved = FALSE;
		     ReallyChanged = TRUE;  }
		  break;
	case 9  : if ( strlen(DateGad_info.Buffer) )
		  {  strcpy(WorkNode.Date,DateGad_info.Buffer);
		     ActivateGadget(&ArtistGad,MainWindow,NULL);
		     Comics.Changed = TRUE;
		     Comics.WorkSaved = FALSE;  }
		  break;
	case 10 : if ( strlen(ArtistGad_info.Buffer) )
		  {  strcpy(WorkNode.Artist,ArtistGad_info.Buffer);
		     ActivateGadget(&CoverGad,MainWindow,NULL);
		     Comics.Changed = TRUE;
		     Comics.WorkSaved = FALSE;  }
		  break;
	case 11 : WorkNode.Cover = afp(CoverGad_info.Buffer);
		  strunc(CoverGad_info.Buffer,WorkNode.Cover);
		  ActivateGadget(&PurchaseGad,MainWindow,NULL);
		  Comics.Changed = TRUE;
		  Comics.WorkSaved = FALSE;
		  break;
	case 12 : /*  if Purchase field contains "=", use cover price */
		  if ( strchr(PurchaseGad_info.Buffer,'=') )
		  	WorkNode.Purchase = WorkNode.Cover;
		  else 
			WorkNode.Purchase = afp(PurchaseGad_info.Buffer);
		  strunc(PurchaseGad_info.Buffer,WorkNode.Purchase);
		  UpdateProfit();
		  ActivateGadget(&CurrentGad,MainWindow,NULL);
		  Comics.Changed = TRUE;
		  Comics.WorkSaved = FALSE;
		  break;
	case 13 : /*  if Current field contains "=", use purchase price */
		  if ( strchr(CurrentGad_info.Buffer,'=') )
		  	WorkNode.Current = WorkNode.Purchase;
		  else
		  	WorkNode.Current = afp(CurrentGad_info.Buffer);
		  strunc(CurrentGad_info.Buffer,WorkNode.Current);
		  UpdateProfit();
		  ActivateGadget(&CopiesGad,MainWindow,NULL);
		  Comics.Changed = TRUE;
		  Comics.WorkSaved = FALSE;
		  break;
	case 14 : WorkNode.Profit = afp(ProfitGad_info.Buffer);
		  UpdateProfit();
		  Comics.Changed = TRUE;
		  Comics.WorkSaved = FALSE;
		  break;
	case 15 : if ( strcmp(WorkNode.Notes,NotesGad_info.Buffer) )
		  {  strcpy(WorkNode.Notes,NotesGad_info.Buffer);
		     Comics.Changed = TRUE;
		     Comics.WorkSaved = FALSE;  } 
		  break;
	case 19 : if ( CopiesGad_info.LongInt )
		  {  WorkNode.Copies = (int) CopiesGad_info.LongInt;
		     ActivateGadget(&NotesGad,MainWindow,NULL);
		     Comics.Changed = TRUE;
		     Comics.WorkSaved = FALSE;   }
		  break;
    }
}

void ActivateNum(code)
USHORT code;
{
	/*  code = rawkey scancode.  If code is one of the digit keys,
	    activate the corresponding string gadget.  */
	    
	switch (code)
	{
		case 0x01 : ActivateGadget(&TitleGad,MainWindow,NULL);
			  break;
		case 0x02 : ActivateGadget(&IssueGad,MainWindow,NULL);
			  break;
		case 0x03 : ActivateGadget(&DateGad,MainWindow,NULL);
			  break;
		case 0x04 : ActivateGadget(&ArtistGad,MainWindow,NULL);
			  break;
		case 0x05 : ActivateGadget(&CoverGad,MainWindow,NULL);
			  break;
		case 0x06 : ActivateGadget(&PurchaseGad,MainWindow,NULL);
			  break;
		case 0x07 : ActivateGadget(&CurrentGad,MainWindow,NULL);
			  break;
		case 0x08 : ActivateGadget(&CopiesGad,MainWindow,NULL);
			  break;
		case 0x09 : ActivateGadget(&NotesGad,MainWindow,NULL);
			  break;
	}
}

	
/*  ENTERING DATA FROM OTHER GADGETS ------------------------ */

void GetFormat()
{
	InputRequester(FIRSTFORMAT,LASTFORMAT,FormatText,"Format");
	RefreshGadgets(&TitleGad,MainWindow,NULL);
	strcpy(WorkNode.Format,FormatText);
}

void GetPublisher()
{
	InputRequester(FIRSTPUB,LASTPUB,PublishText,"Publisher");
	RefreshGadgets(&TitleGad,MainWindow,NULL);
	strcpy(WorkNode.Publisher,PublishText);
}

void GetCondition()
{
	InputRequester(FIRSTCOND,LASTCOND,ConditionText,"Condition");
	RefreshGadgets(&TitleGad,MainWindow,NULL);
	strcpy(WorkNode.Condition,ConditionText);
}


void InputRequester(FirstVal,LastVal,thestring,reqtitle)
USHORT FirstVal,LastVal;
char *thestring,*reqtitle;
{
	ULONG theanswer;
	char inputbuf[20],bodytext[40];
	
	theanswer = BuildRequester(FirstVal,LastVal,reqtitle,TRUE);
	
	if (theanswer > 1)
	{	/* Copy char 3 to end of correct item into thestring  */
		strcpy (thestring,SubItemsText[FirstVal + theanswer - 2].IText + 2);
		Comics.Changed = TRUE;		}
	else if (theanswer == 1)    /*  Enter your own choice */
	{
		strcpy (inputbuf,thestring);
		strcpy (bodytext,"Enter other ");
		strcat (bodytext,reqtitle);
		
		theanswer = 
		        rtGetString(inputbuf,OPTLENGTH,bodytext,NULL,TAG_END);
		
		if (theanswer)
		{	strcpy (thestring,inputbuf);
			/* Add padding spaces */
			while (strlen(thestring) < OPTLENGTH) strcat(thestring," ");
			Comics.Changed = TRUE;    }
	}
}

ULONG BuildRequester(FirstVal,LastVal,reqtitle,InclOther)
USHORT FirstVal,LastVal;
char *reqtitle;
BOOL InclOther;
{
	char bodytext[400];   /* body of requester */
	char stringline[40];  /* each line of the body */
	char gadgettext[40];  /* gadget titles for requester */
	char gadgetname[6];   /* each gadget name */
	USHORT  i;
	ULONG theanswer;
	
	strcpy (bodytext,"Select one by clicking on its button:\n\n");
	if (InclOther)	strcpy (gadgettext,"Other|");
	else		strcpy (gadgettext,"");
	
	for (i=FirstVal;i<=LastVal;i++)
	{
		sprintf(stringline,"    %hu : %s\n",i-FirstVal + 1,SubItemsText[i].IText);
		strcat (bodytext,stringline);
		sprintf(gadgetname,"%hu|",i-FirstVal + 1);
		strcat (gadgettext,gadgetname);
	}
	strcat (gadgettext,"Cancel");
	
	theanswer = rtEZRequestTags(bodytext,gadgettext,NULL,NULL,
				    RTEZ_ReqTitle,reqtitle,
				    RT_ReqPos,REQPOS_CENTERSCR,
				    TAG_END);
	return (theanswer);
}


/*  HANDLING OF MAIN WINDOW'S PROPORTIONAL GADGET ----------- */

void UpdateMainProp()
{
	USHORT NewBody,NewPot;
	
	if (Comics.TotalIssues < 2)
	{
		NewBody = MAXBODY;
		NewPot = 0;
	}
	else
	{
		NewBody = (USHORT) MAXBODY / Comics.TotalIssues;
		/* Subtract 1 from both so at CurIssue = 1, NewPot = 0 */
		NewPot = (USHORT) ( ((ULONG) MAXPOT * (CurIssue-1) ) / (Comics.TotalIssues-1) );
	}
	
	NewModifyProp(&MainProp,MainWindow,NULL,MainProp_info.Flags,
		NewPot,MainProp_info.VertPot,
		NewBody,MainProp_info.VertBody,1L);
}

void TranslateMainProp()
{
	int NewCurIssue;
	
	if (Comics.Changed)	SaveComic();
	
	NewCurIssue = (int) (MainProp_info.HorizPot * Comics.TotalIssues) / MAXPOT;
	
	if (MainProp_info.Flags & KNOBHIT)    /* Actually moved the knob */
	{
		CurIssue = 0;
		current = head;
		while ( (CurIssue++ < NewCurIssue) &&
			(current->next != NULL) )
		{
			current = current->next;
		}
		DisplayComic();		
	}
	else
	{
		if (NewCurIssue < CurIssue)   /* clicked to left of knob */
			PrevTitle();
		else			      /* clicked to right of knob */
			NextTitle();
	}
}


/*  LIST MOVEMENT ROUTINES ---------------------------------- */

void Previous()
{
	if (Comics.Changed)	SaveComic();
	
	if (current != NULL)
	{
		if (current->prev == NULL)	DisplayBeep(ComicScreen);
		else
		{
			previous = current;
			current = current->prev;
			CurIssue--;
			DisplayComic();
		}
	}
	else
		current = previous;    /*  Cancel New Comic  */
}

void Next()
{
	if (Comics.Changed)	SaveComic();
	
	if (current != NULL)
	{
		if (current->next == NULL)	DisplayBeep(ComicScreen);
		else
		{
			previous = current;
			current = current->next;
			CurIssue++;
			DisplayComic();
		}
	}
	else
		current = previous;    /*  Cancel New comic */
}

void First()
{
	if (Comics.Changed)	SaveComic();
	
	previous = current;
	current = head;
	if (current != NULL)
	{
		CurIssue = 1;
		DisplayComic();
	}
}

void Last()
{
	if (Comics.Changed)	SaveComic();
	
	previous = current;
	current = head;
	if (current != NULL)
	{
		while (current->next != NULL)
		{
			current = current->next;
		}
		CurIssue = Comics.TotalIssues;
		DisplayComic();
	}
}

void PrevTitle()
{
	char curtitle[100],testtitle[100];
	
	if (Comics.Changed)	SaveComic();
	
	if (current != NULL)
	{
		previous = current;
		
		/*  Search back as long as haven't reached beginning 
		    of list and the titles are the same   */
		/*  Convert strings so search isn't case sensitive or
		    "the-sensitive"  */

		strcpy (curtitle,WorkNode.Title);
		ConvertStr(curtitle);
		strcpy (testtitle,curtitle);
		
		while ( (current->prev != NULL) &&
			(strcmp(curtitle,testtitle) == 0) )
		{
			current = current->prev;
			CurIssue--;
			strcpy (testtitle,current->Title);
			ConvertStr(testtitle);
		}
		DisplayComic();
	}
}

void NextTitle()
{
	char curtitle[100],testtitle[100];
	
	if (Comics.Changed)	SaveComic();
	
	if (current != NULL)
	{
		previous = current;
		
		/*  Search forward as long as haven't reached beginning 
		    of list and the titles are the same   */
		/*  Convert strings so search isn't case sensitive or
		    "the-sensitive"  */

		strcpy (curtitle,WorkNode.Title);
		ConvertStr(curtitle);
		strcpy (testtitle,curtitle);
		
		while ( (current->next != NULL) &&
			(strcmp(curtitle,testtitle) == 0) )
		{
			current = current->next;
			CurIssue++;
			strcpy (testtitle,current->Title);
			ConvertStr(testtitle);
		}
		DisplayComic();
	}
}

void NewComic()
{
	if (Comics.Changed)	SaveComic();
	
	previous = current;
	ClearNode(&WorkNode);
	current = NULL;
	DisplayComic();
	ActivateGadget(&TitleGad,MainWindow,NULL);
}

void UpdateProfit()
{
	WorkNode.Profit = WorkNode.Current - WorkNode.Purchase;
	strunc (ProfitGad_info.Buffer,WorkNode.Profit);
	RefreshGadgets(&PurchaseGad,MainWindow,NULL);
}


/*  NODE HANDLING FUNCTIONS ----------------------------------------- */

void CopyNode(Ptr1,Ptr2)
ComicNodePtr Ptr1,Ptr2;
/*  Copies information in node pointed to by Ptr2 into position Ptr1.
    Both must have already been created             */
{
	CopyMem((UBYTE *) Ptr2,(UBYTE *) Ptr1,sizeof(struct ComicNode));
}

void ClearNode(Ptr1)
ComicNodePtr Ptr1;
/*  Clears all the information in node pointed to by Ptr1   */
{
	Ptr1->Title[0] = '\0';
	Ptr1->Issue = 0;
	Ptr1->Date[0] = '\0';
	Ptr1->Artist[0] = '\0';
	Ptr1->Cover = 0.0;
	Ptr1->Purchase = 0.0;
	Ptr1->Current = 0.0;
	Ptr1->Profit = 0.0;
	Ptr1->Notes[0] = '\0';
	strcpy (Ptr1->Condition,Comics.defCondition);  /* fill w/defaults */
	strcpy (Ptr1->Publisher,Comics.defPublisher);
	strcpy (Ptr1->Format,Comics.defFormat);
	Ptr1->Copies = Comics.defCopies;
	Ptr1->next = NULL;
	Ptr1->prev = NULL;
}
