/*        COMICS  ORGANIZER                    */
/*        Main Module                          */

#include <exec/types.h>
#include <exec/memory.h>
#include <libraries/dos.h>
#include <intuition/intuition.h>
#include <workbench/startup.h>
#include <proto/exec.h>
#include <proto/dos.h>
#include <graphics/gfxbase.h>
#include <stdio.h>

#include <libraries/reqtools.h>
#include <proto/reqtools.h>

#include "comics.h"

/*   Window/Screen  Interface structures */

struct GfxBase		*GfxBase = NULL;
struct IntuitionBase	*IntuitionBase = NULL;
struct Library		*MathBase = NULL;
struct Library		*IconBase = NULL;
struct ReqToolsBase	*ReqToolsBase = NULL;
struct Screen		*ComicScreen = NULL;
struct Window		*MainWindow = NULL;
struct Window		*IndexWindow = NULL;
struct Window		*BackWindow = NULL;
struct Process		*myProcess;
APTR	oldwinptr;

/*   External Structures                 */

extern struct NewScreen	NewScreen;
extern struct NewWindow	Comic_Window;
extern struct NewWindow	Index_window;
extern struct NewWindow Back_Window;
extern struct Image	titleimage;
extern struct Menu	MenuStrip[];
extern struct MenuItem  Items[];
extern struct Gadget	TitleGad;
extern struct rtFileRequester *filereq;

extern struct ComicFile Comics;

/*   Internal Routines                   */

void OpenAll(), OpenIndex();
void CloseIndex(), CloseAll();
void HandleMessage(struct IntuiMessage *,BOOL);
void DoMenus(USHORT);
void DoGadgets(USHORT);
void DoKeys(USHORT,USHORT), DoIndexKeys(USHORT,USHORT);
void DoHelp();
ULONG HelpAbout(),HelpDefaults(),HelpKeys(),HelpReports(),HelpUpdate();
void NewFile(),QuitIt();
void IssueIndex(), TitleIndex();
void GetColors();


/*   External Routines                    */

extern void DisposeList();
extern void ReadInfo(USHORT);
extern void SaveComic();
extern void Previous(), Next(), First(), Last();
extern void PrevTitle(), NextTitle();
extern void NewComic();
extern void TranslateMainProp();
extern void GetPublisher(), GetFormat(), GetCondition();
extern void SetDefault(char *,USHORT,UWORD);
extern void SetDefCopies();
extern void ChangeMenuItem(UWORD);
extern void PushComic(),PopComic();
extern void FindComic(),FindText();
extern void CutComic(),DeleteComic(),DeleteRange(),PasteComic();
extern void UpdateComic(),UpdateRange();
extern void OpenFile(char *),SaveFile(BOOL);

extern void MakeTitleIndex(),MakeIssueIndex(),DisposeIndex();
extern void IndexScrollUp(),IndexScrollDown(),IndexHome(),ReadIndexProp();
extern void ActivateNum(USHORT),IndexKeyAdvance(char);
extern void GotoIssue(SHORT),GotoTitle(SHORT);
extern void DoReport(USHORT,USHORT);

/*  Other variables                 */

BOOL IssueInd = FALSE;
BOOL TitleInd = FALSE;

extern ComicNodePtr	head,current,previous;
extern BOOL		starthead;
extern FILE	*Fileptr,*Consoleptr;

char BodyText[700];    /*  For the help requesters */
	

/* MAIN PROGRAM ------------------------------- */

void main(argc,argv)
int argc;
char **argv;
{
	ULONG IndexSignal, MainSignal;
	
	extern struct WBStartup *WBenchMsg;
	struct WBArg *wb_arg;
	BPTR olddir;
	
	struct Message	*GetMsg();
	struct IntuiMessage *message1;
	struct IntuiMessage *message2;
	
	
	myProcess = (struct Process *) FindTask (NULL);
	oldwinptr = myProcess->pr_WindowPtr;
	
	OpenAll();
	
	if (argc == 2)	/* Filename given in command line */
		OpenFile(argv[1]);
	else if (argc == 0)  /* run from the workbench */
	{
		if (WBenchMsg->sm_NumArgs >1)
		{   wb_arg = WBenchMsg->sm_ArgList;
		    wb_arg++;	/* advance to first document file */
		    if (wb_arg->wa_Lock != NULL)
		    {	olddir = CurrentDir(wb_arg->wa_Lock);
			OpenFile((char *) wb_arg->wa_Name);
			CurrentDir(olddir);	}
		    else OpenFile((char *) wb_arg->wa_Name);
		}
	}
	
	if (current == NULL) NewComic();
	
	FOREVER
	{
	    message1 = (struct IntuiMessage *) GetMsg(MainWindow->UserPort);
 	    if (IndexWindow)
	    	message2 = (struct IntuiMessage *) GetMsg(IndexWindow->UserPort);
	    else
	    	message2 = NULL;
				
	    if ( (message1 == NULL) && (message2 == NULL) )
	    {
		MainSignal = 1L << MainWindow->UserPort->mp_SigBit;
		if (IndexWindow)
		    IndexSignal = 1L << IndexWindow->UserPort->mp_SigBit;
		else
		    IndexSignal = NULL;
		
		Wait(MainSignal | IndexSignal);
		continue;
	    }    
	
	    /*  Restore title bar if an error message has been printed */
	    if ( strcmp(ComicScreen->Title,"Comics") )
	    	SetWindowTitles(MainWindow,-1,"Comics");
		
	    if (message2)
		HandleMessage(message2,INDEXMSG);
	    if (message1)
		HandleMessage(message1,MAINMSG);
	    
	}  /* End FOREVER */		
}
	
void HandleMessage(messageptr,Flag)
struct IntuiMessage *messageptr;
BOOL Flag;
{
    ULONG class;
    USHORT code,qualifier,GadgetID;
    struct Gadget *GadgetPtr;
    SHORT y;
    
    class = messageptr->Class;
    code = messageptr->Code;
    qualifier = messageptr->Qualifier;
    GadgetPtr = (struct Gadget *) messageptr->IAddress;
    GadgetID = GadgetPtr->GadgetID;
    ReplyMsg( (struct Message *) messageptr);
    
    switch (class)
    {
	case CLOSEWINDOW    : if (Flag == MAINMSG)
					QuitIt();
			      else
			      		CloseIndex();
			      break;
	case ACTIVEWINDOW   : if (Flag == MAINMSG) 
	                      {
				WindowToFront(MainWindow);
			      }
			      else
			      {
				if (IndexWindow) WindowToFront(IndexWindow);
			      }
			      break;
	case MENUPICK       : DoMenus(code);
			      break;
	case GADGETUP	    : DoGadgets(GadgetID);
			      break;
	case GADGETDOWN	    : DoGadgets(GadgetID);
			      break;
	case RAWKEY	    : if (Flag == MAINMSG)
				DoKeys(code,qualifier);
			      else
			      	DoIndexKeys(code,qualifier);
			      break;
	case VANILLAKEY     : /*  Index window got an ASCII key code */
			      IndexKeyAdvance((char) code);
			      break;
	case MOUSEBUTTONS   : /*  Button pressed in Index Window  */
			      if (code & SELECTUP)
			      {
			 	y = (SHORT) messageptr->MouseY;
				if (IssueInd)
					GotoIssue(y);
				else
				    	GotoTitle(y);
			      }
			      break;
    }
}

/*  MENU HANDLING --------------------------------------------- */

void DoMenus(MenuNumber)
UWORD MenuNumber;
{
    UWORD Menu,MenuItem,SubItem;
    struct MenuItem *MenuPoint;
	
    while (MenuNumber != MENUNULL)     /* Repeat until all menu selections
    					  have been processed */
    {
	Menu	 = MENUNUM(MenuNumber);
	MenuItem = ITEMNUM(MenuNumber);
	SubItem  = SUBNUM(MenuNumber);
	
	MenuPoint = (struct MenuItem *) ItemAddress(MenuStrip,MenuNumber);
	
	switch (Menu)
	{
	    case 0 : /* Project menu */
		    switch (MenuItem)
		    {
			case 0: NewFile();
				break;  	/* NEW */
			case 1: OpenFile(NULL);
				/* Set new screen colors */
				LoadRGB4(&(ComicScreen->ViewPort),Comics.ColorMap,4L);
				break;  	/* Open */
			case 2: SaveFile(FALSE);/* Save */
				break;  	/* (no prompt) */
			case 3: SaveFile(TRUE); /* Save as... */
				break;  	/* (prompt for filename) */
			case 5: DoHelp();
				break;  	/* Help */
			case 6: QuitIt();	/* Quit */
			        break;
		    }
		    break;   /* End Comics Menu */
	    case 1 : /* Edit Menu     */
	    	    switch(MenuItem)
		    {
			case 0 : NewComic();
				 if (Comics.AutoUpdate)	UpdateComic();
				 break;
			case 1 : CutComic();
				 break;
			case 2 : DeleteComic();
				 break;
			case 3 : PasteComic();
				 break;
			case 5 : PushComic();
				 break;
			case 6 : PopComic();
				 break;
		    	case 8 : UpdateRange();
				 break;
			case 9 : DeleteRange();
				 break;
			case 11: UpdateComic();
				 break;
		    }
		    break;   /*  End Edit Menu */
	    case 2 : /* Go Menu     */
	    	    switch (MenuItem)
		    {
			case 0 : First();
				 break;
			case 1 : Previous();
				 break;
			case 2 : Next();
				 break;
			case 3 : Last();
				 break;
			case 5 : PrevTitle();
				 break;
			case 6 : NextTitle();
				 break;
			case 8 : FindComic();    /* Find comic  */
				 break;
			case 10: IssueIndex();   /* Show Issue Index */
			         break;
			case 11: TitleIndex();   /* Show Title Index */
				 break;
		    }
		    break;   /*  End Go Menu */
	    case 3 : /* Defaults menu  */
	    	     switch (MenuItem)
		     {
			case 0 : SetDefault(Comics.defPublisher,FIRSTPUB,SubItem);
				 break;
			case 1 : SetDefault(Comics.defCondition,FIRSTCOND,SubItem);
				 break;
			case 2 : SetDefault(Comics.defFormat,FIRSTFORMAT,SubItem);
				 break;
			case 3 : SetDefCopies();
				 break;
			case 4 : ChangeMenuItem(SubItem);
				 break;
			case 6 : switch(SubItem)   /*  Update... */
				 {
					case 0 : Comics.Update ^= UPDATEDATE;
						 break;
					case 1 : Comics.Update ^= UPDATEPRICES;
						 break;
					case 2 : Comics.Update ^= UPDATENOTES;
						 break;
				  }
				  break;
			case 7 : Comics.AutoUpdate = !Comics.AutoUpdate;
				 break;
			case 8 : Comics.ChangeThe = !Comics.ChangeThe;
				 break;
			case 10: GetColors();		/* Change Colors */
				 break;
		    }
		    break;   /* End Defaults menu */
	    case 4 :  /*  Reports Menu  */
	    	     DoReport(MenuItem,SubItem);
		     break;
	}
	MenuNumber = MenuPoint -> NextSelect;
    }
}


/*  GADGET HANDLING ---------------------------------------------- */
void DoGadgets(GadgetID)
USHORT GadgetID;
{
	if ( (GadgetID > 6 && GadgetID < 16) || (GadgetID == 19) )
	/* Is a string gadget */
	{
		ReadInfo(GadgetID);
	}
	else
	{
		switch (GadgetID)
		{
		    case 0 : First();
		    	     break;
		    case 1 : Previous();
		    	     break;
		    case 2 : Next();
		    	     break;
		    case 3 : Last();
		    	     break;
		    case 5 : NewComic();    /* New Comic Gadget */
		    	     if (Comics.AutoUpdate)	UpdateComic();
			     break;
		    case 6 : FindComic();   /* Find Comic Gadget */
		    	     break;
		    case MAINPROPGAD : TranslateMainProp();
		    		       break;
		    case 16 : GetCondition();
			      break;
		    case 17 : GetPublisher();
		    	      break;
		    case 18 : GetFormat();
		    	      break;     
		
			/*  Index window gadgets */
		    case 20 :  /* Up arrow */
		    	      IndexScrollDown();
			      break;
		    case 21 :  /* Down arrow */
		    	      IndexScrollUp();
			      break;
		    case 23 :  /* Update Index button */
		    	      if (IssueInd)	MakeIssueIndex();
			      else		MakeTitleIndex();
			      break;
		    case INDEXPROPGAD : /* Read Index Prop Gad */
		    		        ReadIndexProp();
				        break;      
		
		}
	}
}


/*  KEYBOARD HANDLING  ------------------------------------------- */

void DoKeys(code,qualifier)
USHORT code,qualifier;
{
	switch (code)
	{
		case RETURN	: ActivateGadget(&TitleGad,MainWindow,NULL);
				  break;
		case UP		: First();
				  break;
		case DOWN	: Last();
				  break;
		case LEFT	: if ( (qualifier & IEQUALIFIER_LSHIFT) ||
				       (qualifier & IEQUALIFIER_RSHIFT) )
				       PrevTitle();
				  else
				       Previous();
				  break;
		case RIGHT	: if ( (qualifier & IEQUALIFIER_LSHIFT) ||
				       (qualifier & IEQUALIFIER_RSHIFT) )
				       NextTitle();
				  else
				       Next();
				  break;
		case F1		: PushComic();
				  break;
		case F2		: PopComic();
				  break;
		case F8		: /* Set screen colors */
				  GetColors();
				  break;
		case F9		: IssueIndex();
				  break;
		case F10	: TitleIndex();
				  break;
		case SPACE	: NewComic();
				  if (Comics.AutoUpdate)   UpdateComic();
				  break;
		case UPDATEKEY  : UpdateComic();
				  break;
		case ESC	: if (previous == NULL)	DisplayBeep(ComicScreen);
				  else
				  {
					/*  Back up to last comic seen */
					current = previous;
					DisplayComic();
				  }
				  break;
		case DELETE	: CutComic();
				  break;
		case HELP	: DoHelp();
				  break;
		case FINDKEY	: starthead = FALSE;  /* Always looks for next occurance */
				  FindText();
				  break;
		case TAB	: if (Comics.Changed)	SaveComic();
				  if (current != NULL)  DisplayComic();
				  break;
	}
	/*  Check for SHIFT-Digit keys */
	if ( (code >= NUM1) && (code <= NUM9) )
	{
		ActivateNum(code);
	}
}
			  
void DoIndexKeys(code,qualifier)
USHORT code,qualifier;
{
	switch (code)
	{
		case F9		: IssueIndex();
				  break;
		case F10	: TitleIndex();
				  break;
		case ESC	: CloseIndex();
				  break;
		case UP		: IndexScrollDown();
				  break;
		case DOWN	: IndexScrollUp();
				  break;
		case LEFT	: IndexHome();
				  break;
		case RETURN	: IndexHome();
				  break;
	}
}


/*  COMICS MENU SUBROUTINES -------------------------------------- */

void NewFile()
{
	ULONG response;
	
	if ( !Comics.WorkSaved || Comics.Changed )
	{
		response = rtEZRequestTags("Erase all entries?","NEW|Cancel",NULL,NULL,	
					    RTEZ_ReqTitle,"FILE NOT SAVED",
					    TAG_END);
	}
	else	response = TRUE;
	
	if (response)
	{
		DisposeList();
		NewComic();
	}
}
						
void QuitIt()
{
	UBYTE	*bodytext;
	ULONG	response;
		
	if ( Comics.WorkSaved && !Comics.Changed )
	{
		bodytext = (UBYTE *) "QUIT: Are you sure?";
		response = rtEZRequestTags(bodytext,"Quit|Cancel",NULL,NULL,
					   RTEZ_ReqTitle,"Quit Program",
					   TAG_END);
	}
	else	
	{
		bodytext = (UBYTE *) "DATA IS NOT SAVED: Are you sure?";
		response = rtEZRequestTags(bodytext,"Quit|Save...|Cancel",NULL,NULL,
					   RTEZ_ReqTitle,"Quit Program",
					   RTEZ_DefaultResponse, 2,
					   TAG_END);
	}
	
	if ( response == TRUE )
	{
		CloseAll();
		exit(TRUE);
	}
	else if (response == 2)
	{
		SaveFile(FALSE);
		if (Comics.WorkSaved)
		{	CloseAll();
			exit(TRUE);     }
	}
	else
		ActivateWindow(MainWindow);
}

	
void GetColors()
{
	struct Screen ScreenData;
	register int i;
	
	rtPaletteRequest ("Set Colors:",NULL,
			  RT_ReqPos,REQPOS_CENTERSCR,TAG_END);
	
	if (GetScreenData(&ScreenData,sizeof(ScreenData),CUSTOMSCREEN,ComicScreen))
	{
		for (i=0;i<4;i++)
			Comics.ColorMap[i] = GetRGB4((ScreenData.ViewPort.ColorMap),i);
	}
}
	

/*  GO MENU SUBROUTINES --------------------------------------- */

void IssueIndex()
{
	if (IssueInd)   /*  If Issue Index is already visible, close it */
	{
		CloseIndex();
	}
	else
	{
		if (head == NULL)	return;  /* Ignore if empty list */
		
		if (!(IndexWindow)) OpenIndex(); /* Open index window if
						   it's not already open */
		IssueInd = TRUE;
		TitleInd = FALSE;

		/*  Set check marks in menu */
		Items[ISSUEMENU].Flags = ITEMENABLED+HIGHCOMP+CHECKIT+ITEMTEXT+MENUTOGGLE+CHECKED;
		Items[TITLEMENU].Flags = ITEMENABLED+HIGHCOMP+CHECKIT+ITEMTEXT+MENUTOGGLE;
		ClearMenuStrip(MainWindow);
		ClearMenuStrip(IndexWindow);
		SetMenuStrip(MainWindow,MenuStrip);
		SetMenuStrip(IndexWindow,MenuStrip);

		/* RefreshIssueIndex; */
		SetWindowTitles(IndexWindow,(UBYTE *) "Issue Index",-1);
		ActivateWindow(IndexWindow);
		
		MakeIssueIndex();
	}
}

void TitleIndex()
{
	if (TitleInd)   /*  If Title index already visible, close it */
	{
		CloseIndex();
	}
	else
	{
		if (head == NULL)	return;  /* Ingore if empty list */
		
		if (!(IndexWindow)) OpenIndex();  /* open window if it's
						     not already open   */
		IssueInd = FALSE;
		TitleInd = TRUE;
		
		/*  Set check marks in menu */
		Items[ISSUEMENU].Flags = ITEMENABLED+HIGHCOMP+CHECKIT+ITEMTEXT+MENUTOGGLE;
		Items[TITLEMENU].Flags = ITEMENABLED+HIGHCOMP+CHECKIT+ITEMTEXT+MENUTOGGLE+CHECKED;
		ClearMenuStrip(MainWindow);
		ClearMenuStrip(IndexWindow);
		SetMenuStrip(MainWindow,MenuStrip);
		SetMenuStrip(IndexWindow,MenuStrip);
		
		/* RefreshTitleIndex; */
		SetWindowTitles(IndexWindow,(UBYTE *) "Title Index",-1);
		ActivateWindow(IndexWindow);
		
		MakeTitleIndex();
	}
}


/*  OPEN/CLOSE SUBROUTINES ------------------------------------ */

void OpenAll()
{
	if (!(IntuitionBase = (struct IntuitionBase *)
		OpenLibrary("intuition.library",0L)))
		{
			printf("Cannot open intuition library!\n");
			CloseAll();
			exit(FALSE);
		}
	if (!(GfxBase = (struct GfxBase *)
		OpenLibrary("graphics.library",0L)))
		{
			printf("Cannot open graphics library!\n");
			CloseAll();
			exit(FALSE);
		}
	
	if (!(ComicScreen = (struct Screen *)
		OpenScreen(&NewScreen)))
		{
			printf("Cannot open main screen\n");
			CloseAll();
			exit(FALSE);
		}
	
	LoadRGB4(&(ComicScreen->ViewPort),Comics.ColorMap,4L);  /* Set screen colors */
	
	Comic_Window.Screen = ComicScreen;
	Index_window.Screen = ComicScreen;
	Back_Window.Screen = ComicScreen;
	
	BackWindow = (struct Window *) OpenWindow(&Back_Window);
	if (BackWindow) DrawImage(BackWindow->RPort,&titleimage,0L,0L);
	
	if (!(MainWindow = (struct Window *)
		OpenWindow(&Comic_Window)))
		{
			printf("Cannot open main window!\n");
			CloseAll();
			exit(FALSE);
		}
	myProcess->pr_WindowPtr = (APTR) MainWindow;
	SetMenuStrip(MainWindow,MenuStrip);
		
	if (!(ReqToolsBase = (struct ReqToolsBase *)
		OpenLibrary (REQTOOLSNAME, REQTOOLSVERSION)))
	{
		printf("Can't find reqtools.library V37 (or higher)!\n");
		CloseAll();
		exit(FALSE);
	}
	
	if (!(MathBase = OpenLibrary("mathffp.library",0L)))
	{	
		rtEZRequest("Can't find mathffp.library!","EXIT",NULL,NULL);
		CloseAll();
		exit(FALSE);
	}
	
	if (!(IconBase = OpenLibrary("icon.library",0L)))
	{
		rtEZRequest("Can't open icon.library.\nNo project icons will be saved.","OK",NULL,NULL);
	}	
}


void OpenIndex()
{
	ULONG response = TRUE;

	do 
	{
		
	if (!(IndexWindow = (struct Window *)
		OpenWindow(&Index_window)))
		{
			response = rtEZRequest("Can't open Index Window!","Retry|Cancel",NULL,NULL);
		}
	else
		{
			response = FALSE;
			SetMenuStrip(IndexWindow,MenuStrip);
		}
	
	}  while (response == TRUE);
}

void CloseIndex()
{
	if (IndexWindow)
	{
		ClearMenuStrip(IndexWindow);
		CloseWindow(IndexWindow);
	}
	TitleInd = FALSE;
	IssueInd = FALSE;
	IndexWindow = NULL;
	DisposeIndex();
	
	/*  Clear check marks from menu */
	Items[ISSUEMENU].Flags = ITEMENABLED+HIGHCOMP+CHECKIT+ITEMTEXT+MENUTOGGLE;
	Items[TITLEMENU].Flags = ITEMENABLED+HIGHCOMP+CHECKIT+ITEMTEXT+MENUTOGGLE;
	ClearMenuStrip(MainWindow);
	SetMenuStrip(MainWindow,MenuStrip);
}

void CloseAll()
{
	DisposeList();
	if (filereq)	rtFreeRequest((APTR) filereq);
	
	myProcess->pr_WindowPtr = oldwinptr;
	
	if (Fileptr)	fclose(Fileptr);
	if (Consoleptr)	fclose(Consoleptr);
	if (IndexWindow)
	{
		ClearMenuStrip(IndexWindow);
		CloseWindow(IndexWindow);
	}
	if (MainWindow)
	{
		ClearMenuStrip(MainWindow);
		CloseWindow(MainWindow);
	}
	if (BackWindow)		CloseWindow(BackWindow);
	if (ComicScreen)	CloseScreen(ComicScreen);
	if (IconBase)		CloseLibrary(IconBase);
	if (MathBase)		CloseLibrary(MathBase);
	if (ReqToolsBase)	CloseLibrary( (struct Library *) ReqToolsBase);
	if (GfxBase)		CloseLibrary( (struct Library *) GfxBase);
	if (IntuitionBase)	CloseLibrary(IntuitionBase);	
	
	OpenWorkBench();
}

/* HELP ROUTINES ----------------------------------------------- */

void DoHelp()
{
	ULONG response;
	
	response = HelpAbout();
	
	while ( response )
	{
		switch(response)
		{
			case 1 : response = HelpAbout();
				 break;
			case 2 : response = HelpKeys();
				 break;
			case 3 : response = HelpUpdate();
				 break;
			case 4 : response = HelpDefaults();
				 break;
			case 5 : response = HelpReports();
				 break;
		}
	}
}

ULONG HelpAbout()
{
	strcpy (BodyText,"Amiga Comics Organizer\n\n");
	strcat (BodyText,"by Chuck Jordan\nUSENET: jordan@castor.cs.uga.edu\n\n");
	strcat (BodyText,"requesters: ReqTools.library V37\n(c) 1991 Nico Franois\n");
	strcat (BodyText,"source: created with 'PowerSource'\nby Jan van den Baard\n(c) 1990-91 by Jaba Development\n");
	strcat (BodyText,"\nClick on one of the buttons below for more help");

	return ( rtEZRequestTags(BodyText,"About|Keys|Update|Defaults|Reports|END",NULL,NULL,
				 RT_ReqPos,REQPOS_CENTERSCR,
				 RTEZ_ReqTitle,"About...",
				 RTEZ_Flags,EZREQF_LAMIGAQUAL | EZREQF_CENTERTEXT,
				 RTEZ_DefaultResponse,2L,
				 TAG_END) );
}

ULONG HelpKeys()
{
	strcpy (BodyText,"CURSOR:\n");
	strcat (BodyText,"LEFT/RIGHT  Previous/Next     UP/DOWN  First/Last\n");
	strcat (BodyText,"      SHIFT-LEFT/RIGHT  Previous/Next Title\n\n");
	strcat (BodyText,"OTHER:\n");
	strcat (BodyText,"DEL         Delete Comic      HELP     This help requester\n");
	strcat (BodyText,"F1/F2       Push/Pop Comic    F9/F10   Title/Issue Index\n");
	strcat (BodyText,"SPACE       New Comic         ESC      Cancel New\n");
	strcat (BodyText," `          Update Comic      F8       Change Colors\n");
	strcat (BodyText," f          Find next         TAB      Enter this comic\n\n");
	strcat (BodyText,"IN FIELDS:\n");
	strcat (BodyText," `  anywhere in title field updates comic.\n");
	strcat (BodyText," =  in Purchase field uses the Cover price field's info.\n");
	strcat (BodyText," =  in Current field uses Purchase price field's info.");	

	return ( rtEZRequestTags(BodyText,"About|Keys|Update|Defaults|Reports|END",NULL,NULL,
				 RT_ReqPos,REQPOS_CENTERSCR,
				 RTEZ_ReqTitle,"Keys",
				 RTEZ_Flags,EZREQF_LAMIGAQUAL,
				 RTEZ_DefaultResponse,3L,
				 TAG_END) );
}

ULONG HelpUpdate()
{
	strcpy (BodyText,"Use the (`) key or enter a (`) in the title field\n");
	strcat (BodyText,"to update this comic.  Uses the previous issue's information.\n\n");
	strcat (BodyText,"Date must be in the format  MON YR, e.g.  JUN 71\n\n");
	strcat (BodyText,"If Auto-Update in the Defaults menu is checked, each\n");
	strcat (BodyText,"new comic that is created will be updated with\n");
	strcat (BodyText,"the previous comic's information.\n\n");
	strcat (BodyText,"Use the Update submenu in the Defaults menu to set which\n");
	strcat (BodyText,"fields are included when updating comics, Notes, Date, & Prices.");

	return ( rtEZRequestTags(BodyText,"About|Keys|Update|Defaults|Reports|END",NULL,NULL,
				 RT_ReqPos,REQPOS_CENTERSCR,
				 RTEZ_ReqTitle,"Update",
				 RTEZ_Flags,EZREQF_LAMIGAQUAL,
				 RTEZ_DefaultResponse,4L,
				 TAG_END) );
}

ULONG HelpDefaults()
{
	strcpy (BodyText,"The selections you check in the Publisher, Condition, and\n");
	strcat (BodyText,"Format submenus will automatically be placed in these fields\n");
	strcat (BodyText,"when you create a New comic.\n\n");
	strcat (BodyText,"If the publisher, condition, or format you want is not\n");
	strcat (BodyText,"in these submenus, select Change Item... from the defaults\n");
	strcat (BodyText,"menu to actually change one of the submenus.\n\n");
	strcat (BodyText,"Select 'Copies' from this menu to set the default number\n");
	strcat (BodyText,"of copies.\n\n");
	strcat (BodyText,"If you select 'Change The' in the defaults menu, titles with\n");
	strcat (BodyText,"'The' as the first word in the title will be sorted\n");
	strcat (BodyText,"correctly, automatically.  Note that this doesn't change\n");
	strcat (BodyText,"the titles you've entered.");

	return ( rtEZRequestTags(BodyText,"About|Keys|Update|Defaults|Reports|END",NULL,NULL,
				 RT_ReqPos,REQPOS_CENTERSCR,
				 RTEZ_ReqTitle,"Defaults",
				 RTEZ_Flags,EZREQF_LAMIGAQUAL | EZREQF_CENTERTEXT,
				 RTEZ_DefaultResponse,5L,
				 TAG_END) );
}

ULONG HelpReports()
{
	strcpy (BodyText,"Totals:  prints out total # of issues, cost, value, & profit.\n");
	strcat (BodyText,"Most Valuable/Profitable:  prints out a sorted list of your\n");
	strcat (BodyText,"                           most valuable or profitable comics.\n");
	strcat (BodyText,"Oldest:  sorted list of oldest comics (sorted by year ONLY)\n");
	strcat (BodyText,"Distribution:  shows number of issues for publishers, conditions,\n");
	strcat (BodyText,"               formats, or found text in Artist or Notes fields.\n");
	strcat (BodyText,"Missing Issues:  prints issues missing from a series, using the\n");
	strcat (BodyText,"                 starting & ending issues you supply.\n");
	strcat (BodyText,"ALL REPORTS are printed to a text file, which can then\n");
	strcat (BodyText,"be edited, formatted, and printed from almost any text editor.");

	return ( rtEZRequestTags(BodyText,"About|Keys|Update|Defaults|Reports|END",NULL,NULL,
				 RT_ReqPos,REQPOS_CENTERSCR,
				 RTEZ_ReqTitle,"Reports",
				 RTEZ_Flags,EZREQF_LAMIGAQUAL,
				 RTEZ_DefaultResponse,1L,
				 TAG_END) );
}
